"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Service_1 = require("frontblock-generic/Service");
const Logger = require("log4js");
const logger = Logger.getLogger(); // logs to STDOUT  
logger.level = 'debug';
const bsock = require('bsock');
/**
 * Frontblock api connection lib
 *
 * Can be used solo and also is a valid frontblock plugin on the customer-side
 */
class FrontblockApiClient {
    constructor(conf) {
        this.apikey = "";
        this.started = false;
        if (conf == null) {
            conf = {
                apiHost: "localhost",
                apiKey: "",
                apiPort: 10000,
                tls: false
            };
            this.conf = conf;
        }
        else {
            if (conf.apiKey != null)
                this.apikey = conf.apiKey;
            this.conf = conf;
            this.start();
        }
    }
    exportRPCs() {
        const unhookFn = async (uid) => { return await this.unsubscribe(this.apikey, uid); };
        return [
            {
                name: "subscribe",
                rpc: async (coin, account, callback) => { return await this.subscribe(this.apikey, coin, account, callback); },
                type: 'hook',
                unhook: unhookFn,
                visibility: 'private'
            }, {
                name: 'unsubscribe',
                rpc: unhookFn,
                type: 'unhook',
                visibility: 'private'
            }
        ];
    }
    start() {
        if (this.started) {
            logger.warn("FrontblockApiClient has already been started. Ignoring");
            return;
        }
        logger.info("Starting apiClient with ", this.conf);
        this.socket = bsock.connect(this.conf.apiPort, this.conf.apiHost, this.conf.tls != null ? this.conf.tls : false);
        this.started = true;
    }
    stop() {
        if (!this.started) {
            logger.warn("FrontblockApiClient has not been started. Ignoring");
            return;
        }
        this.socket.close();
    }
    async subscribe(apikey, coin, account, callback) {
        const r = await this.socket.call("subscribe", apikey, coin, account);
        const res = Service_1.parseSubResponse(r);
        if (res instanceof Service_1.SubscriptionResponse) {
            this.socket.hook(res.uid, callback);
        }
        return res;
    }
    async unsubscribe(apikey, uid) {
        const r = await this.socket.call('unsubscribe', apikey, uid);
        const res = Service_1.parseResponse(r);
        if (res instanceof Service_1.SuccessResponse)
            this.socket.unhook(uid);
        return res;
    }
    async getPluginList() {
        return await this.socket.call('getPluginList');
    }
}
exports.default = FrontblockApiClient;
